/*
* This file is part of fwmon
* Copyright (c) 2001 Gianni Tedesco
*
*    fwmon is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    fwmon is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with fwmon; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "slog.h"

static char buf[BUFFER_SIZE];
char *b=(char *)&buf;
int pos=0;
static const char rcsid[]=
	"@(#) $Header: /data/cvsroot/fwmon/slog.c,v 1.2 2001/12/20 11:27:53 gianni Exp $";

int slog_init(void)
{
	if ( flag(FLAG_SYSLOG) ) {
		openlog("fwmon", LOG_CONS|LOG_PID|LOG_NDELAY, LOG_DAEMON);
	}
	return 1;
}

void slog_close(void)
{
	if ( flag(FLAG_SYSLOG) ) {
		closelog();
	}
}

int slog_addpair(const char *key, const char *format, ...)
{
	static char tmpbuf[BUFFER_SIZE];
	char *t=(char *)&tmpbuf;
	int len[2];
	va_list va;
	
	memset(t, 0, BUFFER_SIZE);	

	/* Format the value*/
	va_start(va, format);
	vsnprintf(t, BUFFER_SIZE-1, format, va);
	va_end(va);

	/* Compute lengths */
	len[0]=strlen(key);
	len[1]=strlen(t);

	/* Does this keypair fit on our buffer? */
	if ( pos+len[0]+len[1]+2 >= BUFFER_SIZE )
		return 0;
	
	strncat(b, key, len[0]);
	strncat(b, "=", 1);
	strncat(b, t, len[1]);
	strncat(b, " ", 1);

	pos+=len[0]+len[1]+2;

	return 1;
}

void slog_dump ( p_packet p )
{
	static char sip[128], dip[128];
	char *src_ip=(char *)&sip;
	char *dst_ip=(char *)&dip;
	
	struct tcphdr  *tcp = p->trans;
	struct udphdr  *udp = p->trans;
	struct icmphdr *icmp = p->trans;
	int sport=0, dport=0;

	
	pos=0;
	memset(b, 0, BUFFER_SIZE);
	memset(src_ip, 0, 128);
	memset(dst_ip, 0, 128);
	
	format_ip(p->ip->saddr, src_ip);
	format_ip(p->ip->daddr, dst_ip);
	
	/* set ports, dependant on protocol */
	switch ( p->ip->protocol ){
		case IPPROTO_TCP:
			sport=ntohs(tcp->source);
			dport=ntohs(tcp->dest);
			break;
		case IPPROTO_UDP:
			sport=ntohs(udp->source);
			dport=ntohs(udp->dest);
			break;
		case IPPROTO_ICMP:
			sport=icmp->type;
			dport=icmp->code;
			break;
	}

	
	/* Log interface and IP header stuff */
	slog_addpair("IN",   "%s", p->data.netlink.iface);
	slog_addpair("SRC",  "%s", src_ip);
	slog_addpair("DST",  "%s", dst_ip);
	slog_addpair("PROTO","%s", get_proto(p->ip->protocol));
	slog_addpair("LEN","%i", p->ip->ttl);
	slog_addpair("TTL","%i", ntohs(p->ip->tot_len));

	switch ( p->ip->protocol ) {
		case IPPROTO_TCP:
			slog_addpair("TCP_FLAGS","%s", get_tcpflags(tcp));
		case IPPROTO_UDP:
			slog_addpair("DPT","%i", dport);
			slog_addpair("SPT","%i", sport);
			break;
		case IPPROTO_ICMP:
			slog_addpair("ICMP_TYPE","%i", sport);
			slog_addpair("ICMP_CODE","%i", dport);
			break;
	}

	syslog(LOG_WARNING, b);
}
